cdef class Frame:

    def __init__(self, name: str):
        self.lexicalUnits = []
        self.frameElements = []
        self.name = name

    cpdef addLexicalUnit(self, str lexicalUnit):
        self.lexicalUnits.append(lexicalUnit)

    cpdef addFrameElement(self, str frameElement):
        self.frameElements.append(frameElement)

    cpdef bint lexicalUnitExists(self, str synSetId):
        return synSetId in self.lexicalUnits

    cpdef str getLexicalUnit(self, int index):
        return self.lexicalUnits[index]

    cpdef str getFrameElement(self, int index):
        return self.frameElements[index]

    cpdef int lexicalUnitSize(self):
        return len(self.lexicalUnits)

    cpdef int frameElementSize(self):
        return len(self.frameElements)

    cpdef str getName(self):
        return self.name



cdef class FrameElement(object):

    def __init__(self, frameElement: str):
        """
        A constructor of FrameElement class which takes frameElement string which is in the form of frameElementType$id
        and parses this string into frameElementType and id. If the frameElement string does not contain '$' then the
        constructor return a NONE type frameElement.

        PARAMETERS
        ----------
        frameElement : str
            Argument string containing the argumentType and id
        """
        if "$" in frameElement:
            self.__frameElementType = frameElement[0:frameElement.index("$")]
            self.__frame = frameElement[frameElement.index("$") + 1:frameElement.rindex("$")]
            self.__id = frameElement[frameElement.rindex("$") + 1:]
        else:
            self.__frameElementType = "NONE"

    cpdef initWithId(self, str frameElementType, str frame, str _id):
        """
        Another constructor of FrameElement class which takes frameElementType and id as inputs and initializes corresponding
        attributes

        PARAMETERS
        ----------
        frameElementType : str
            Type of the argument
        _id : str
            Id of the argument
        """
        self.__frameElementType = frameElementType
        self.__frame = frame
        self.__id = _id

    cpdef str getFrameElementType(self):
        """
        Accessor for frameElementType.

        RETURNS
        -------
        str
            frameElementType.
        """
        return self.__frameElementType

    cpdef str getFrame(self):
        """
        Accessor for frame.

        RETURNS
        -------
        str
            frame.
        """
        return self.__frame

    cpdef str getId(self):
        """
        Accessor for id.

        RETURNS
        -------
        str
            id.
        """
        return self.__id

    def __str__(self)-> str:
        """
        __str__ converts an Argument to a string. If the frameElementType is "NONE" returns only "NONE", otherwise
        it returns frameElement string which is in the form of frameElementType$id

        RETURNS
        -------
        str
            string form of argument
        """
        if self.__frameElementType == "NONE":
            return self.__frameElementType
        else:
            return self.__frameElementType + "$" + self.__frame + "$" + self.__id



cdef class FrameElement(object):

    cdef str __frameElementType
    cdef str __frame
    cdef str __id

    cpdef initWithId(self, str frameElementType, str frame, str _id)
    cpdef str getFrameElementType(self)
    cpdef str getFrame(self)
    cpdef str getId(self)



from FrameNet.Frame cimport Frame


cdef class FrameNet:

    cdef list frames

    cpdef bint lexicalUnitExists(self, str synSetId)
    cpdef list getFrames(self, str synSetId)
    cpdef int size(self)
    cpdef Frame getFrame(self, int index)



cdef class Frame:

    cdef str name
    cdef list lexicalUnits
    cdef list frameElements

    cpdef addLexicalUnit(self, str lexicalUnit)
    cpdef addFrameElement(self, str frameElement)
    cpdef bint lexicalUnitExists(self, str synSetId)
    cpdef str getLexicalUnit(self, int index)
    cpdef str getFrameElement(self, int index)
    cpdef int lexicalUnitSize(self)
    cpdef int frameElementSize(self)
    cpdef str getName(self)



from setuptools import setup

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()
from Cython.Build import cythonize

setup(
    ext_modules=cythonize(["FrameNet/*.pyx"],
                          compiler_directives={'language_level': "3"}),
    name='NlpToolkit-FrameNet-Cy',
    version='1.0.5',
    packages=['FrameNet'],
    package_data={'FrameNet': ['*.pxd', '*.pyx', '*.c', '*.py']},
    url='https://github.com/StarlangSoftware/TurkishFrameNet-Cy',
    license='',
    author='olcaytaner',
    author_email='olcay.yildiz@ozyegin.edu.tr',
    description='FrameNet library',
    long_description=long_description,
    long_description_content_type='text/markdown'
)



import os
import xml.etree.ElementTree


cdef class FrameNet:

    def __init__(self):
        self.frames = []
        root = xml.etree.ElementTree.parse("../framenet.xml").getroot()
        for frameNode in root:
            frame = Frame(frameNode.attrib["NAME"])
            for childNode in frameNode:
                if childNode.tag == "LEXICAL_UNITS":
                    for lexicalUnit in childNode:
                        frame.addLexicalUnit(lexicalUnit.text)
                elif childNode.tag == "FRAME_ELEMENTS":
                    for frameElement in childNode:
                        frame.addFrameElement(frameElement.text)
            self.frames.append(frame)

    cpdef bint lexicalUnitExists(self, str synSetId):
        cdef Frame frame
        for frame in self.frames:
            if frame.lexicalUnitExists(synSetId):
                return True
        return False

    cpdef list getFrames(self, str synSetId):
        cdef list result
        cdef Frame frame
        result = []
        for frame in self.frames:
            if frame.lexicalUnitExists(synSetId):
                result.append(frame)
        return result

    cpdef int size(self):
        return len(self.frames)

    cpdef Frame getFrame(self, int index):
        return self.frames[index]
